@@ -1,13 +1,11 @@
+import os
 import requests
 from ._utils.utils import load_credentials
 
 # Optional params: start_time,end_time,since_id,until_id,max_results,next_token,
 # expansions,tweet.fields,media.fields,poll.fields,place.fields,user.fields
-query_params = {'query': '(from:twitterdev -is:retweet) OR #twitterdev',
-                'tweet.fields': 'author_id'}
 
-
-KEYS = load_credentials('credentials/twitter_keys.yaml')
+KEYS = {}
 
 def bearer_oauth(r):
     r.headers["Authorization"] = f"Bearer {KEYS['bearer_token']}"
@@ -15,10 +13,17 @@ def bearer_oauth(r):
     return r
 
 
-def retrieve_tweets(query_params):
+def retrieve_tweets(query_params, keys):
     search_url = "https://api.twitter.com/2/tweets/search/all"
+    global KEYS
+    try:
+        KEYS = load_credentials(keys)
+    except Exception as e:
+        try:
+            KEYS['bearer_token'] = os.environ[keys]
+        except KeyError:
+            KEYS['bearer_token'] = keys
     response = requests.request("GET", search_url, auth=bearer_oauth, params=query_params)
-    print(response.status_code)
     if response.status_code != 200:
         raise Exception(response.status_code, response.text)
     return response.json()
