@@ -12,14 +12,13 @@ from ._utils.utils import *
 import seaborn as sns
 import matplotlib.pyplot as plt
 from datetime import datetime, timedelta
-#from searchtweets import collect_results, load_credentials
+# from searchtweets import collect_results, load_credentials
 from .geomethods import TweetGeoGenerator
 from .apicall import retrieve_tweets
 
 from wordcloud import WordCloud, STOPWORDS
 
 
-
 class TweetDownloader:
     """Tweet downloading class.
     The TweetDownloader class contains the main downloading function
@@ -73,11 +72,17 @@ class TweetDownloader:
         timestamp
     """
 
-    def __init__(self, credentials,
-                 name='Project_{}'.format(datetime.now().strftime('%m%d%Y_%H%M%S')),
+    def __init__(self,
+                 yaml_credentials=None,
+                 env_token=None,
+                 bearer_token=None,
+                 name=None,
                  output_folder=''):
-        self.name = name
-        self.credentials = credentials
+        if name == None:
+            self.name = 'Project_{}'.format(datetime.now().strftime('%m%d%Y_%H%M%S'))
+        else:
+            self.name = name
+        self.credentials = yaml_credentials
         self.output_folder = output_folder
         self.tweets = None
         self.authors = None
@@ -87,8 +92,9 @@ class TweetDownloader:
         self.authors_df = None
         self.places_df = None
         self.replies_df = None
-        #self.search_args = load_credentials(keys_path=self.credentials)
         self.timestamp = None
+        self.env_token = env_token
+        self.bearer_token = bearer_token
 
     def tweets_from_query(self, query_params, max_page, save_temp, max_tweets, reply_mode=False):
         # initializes a list to store retrieved tweet pages
@@ -112,10 +118,19 @@ class TweetDownloader:
             # Collect results according to query parameters, tweets per page...
             # ... and authentication credentials
 
-            #tweets_page = collect_results(query_params, max_tweets=max_page,
+            # tweets_page = collect_results(query_params, max_tweets=max_page,
             #                              result_stream_args=self.search_args)
 
-            tweets_page = retrieve_tweets(query_params=query_params)
+            if self.credentials != None:
+                keys = self.credentials
+            elif self.env_token != None:
+                keys = self.env_token
+            elif self.bearer_token != None:
+                keys = self.bearer_token
+            else:
+                raise AttributeError('No authentication keys provided')
+
+            tweets_page = retrieve_tweets(query_params=query_params, keys=keys)
 
             if len(tweets_page) != 0:  # ensures we don't process a blank page
 
@@ -124,8 +139,10 @@ class TweetDownloader:
 
                 # Adds number of retrieved tweets to tweet count
                 tweet_count += tweets_page['meta']['result_count']
-
-                df_page = pd.DataFrame(tweets_page['data'])
+                try:
+                    df_page = pd.DataFrame(tweets_page['data'])
+                except KeyError:
+                    continue
                 df_page_authors = pd.DataFrame(tweets_page['includes']['users'])
 
                 df_tweets = pd.concat([df_tweets, df_page])
@@ -192,9 +209,9 @@ class TweetDownloader:
         return list_tweet_pages, df_tweets, df_places, df_authors
 
     def get_tweets(self, query,
-                   start_time=(datetime.now() - timedelta(days=1)).strftime("%Y-%m-%dT%H:%M:%Sz"),
-                   end_time=datetime.now().strftime("%Y-%m-%dT%H:%M:%SZ"),
-                   lang=None, include_retweets=False, place=None,
+                   start_time=None,
+                   end_time=None,
+                   lang=None, include_retweets=False, place=None, has_geo=True,
                    max_tweets=10, max_page=500, save_temp=True, save_final=True,
                    save_replies=False, include_replies=False, max_replies=10, temp_replies=True):
         """
@@ -212,6 +229,8 @@ class TweetDownloader:
             Whether to include tweets that are just a retweet of a previous one (default is False)
         place : str, optional
             Two letter code for country or place in which the search is going to be constraint
+        has_geo : bool, optional
+            Whether to only include tweets with geographic reference (default is True)
         max_tweets : int
             The maximum amount of tweets to retrieve in total (default is 10)
         max_page : int
@@ -228,9 +247,12 @@ class TweetDownloader:
             Whether to save progress while downloading replies if these are allowed (default is True)
         """
 
+        if start_time == None:
+            start_time = validate_date((datetime.now() - timedelta(days=2)).strftime("%Y-%m-%dT%H:%M:%Sz"))
+        if end_time == None:
+            end_time = validate_date((datetime.now() - timedelta(days=1)).strftime("%Y-%m-%dT%H:%M:%Sz"))
 
         # Query parameters
-        has_geo = True  # This could be introduced as func parameter in a future version
         query = '({})'.format(query)
         # Creates a timestamp to avoid overwriting old files
         self.timestamp = datetime.now().strftime('%m%d%Y_%H%M%S.csv')
@@ -265,18 +287,21 @@ class TweetDownloader:
 
         print(f'Tweets download done. A total of {self.tweets_df.shape[0]} tweets were retrieved.')
 
-        if save_final:
+        try:
             self.tweets_df['place_id'] = self.tweets_df.geo.apply(lambda x: get_attribute_from_dict(x, 'place_id'))
-            # Creates date column in date format
-            self.tweets_df['date'] = pd.to_datetime(self.tweets_df.created_at)
-            self.tweets_df['date'] = self.tweets_df.date.dt.strftime('%m/%d/%Y %H:%M%:%S')
-            self.tweets_df['date'] = pd.to_datetime(self.tweets_df['date'])
+        except AttributeError:
+            self.tweets_df['place_id'] = np.nan
+        # Creates date column in date format
+        self.tweets_df['date'] = pd.to_datetime(self.tweets_df.created_at)
+        self.tweets_df['date'] = self.tweets_df.date.dt.strftime('%m/%d/%Y %H:%M:%S')
+        self.tweets_df['date'] = pd.to_datetime(self.tweets_df['date'], utc=False, format='%m/%d/%Y %H:%M:%S')
 
-            # Get metrics as separate columns:
-            self.tweets_df['likes'] = self.tweets_df.public_metrics.apply(lambda x: x['like_count'])
-            self.tweets_df['replies'] = self.tweets_df.public_metrics.apply(lambda x: x['reply_count'])
-            self.tweets_df['retweets'] = self.tweets_df.public_metrics.apply(lambda x: x['retweet_count'])
+        # Get metrics as separate columns:
+        self.tweets_df['likes'] = self.tweets_df.public_metrics.apply(lambda x: x['like_count'])
+        self.tweets_df['replies'] = self.tweets_df.public_metrics.apply(lambda x: x['reply_count'])
+        self.tweets_df['retweets'] = self.tweets_df.public_metrics.apply(lambda x: x['retweet_count'])
 
+        if save_final:
             # saving final dataframes
 
             self.tweets_df.to_csv(f'{filename}_tweets_{self.timestamp}', index=False)
@@ -339,7 +364,8 @@ class TweetDownloader:
 
             query_params = {'query': query,
                             'start_time': '2007-01-01T00:00:00z',
-                            'end_time': datetime.now().strftime("%Y-%m-%dT%H:%M:%SZ"),
+                            'end_time': validate_date(
+                                (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%dT%H:%M:%Sz")),
                             'expansions': 'author_id',
                             'tweet.fields': 'created_at,author_id,id,conversation_id',
                             'user.fields': 'id,name,username,public_metrics',
@@ -354,8 +380,8 @@ class TweetDownloader:
             total_replies = df_tweets_rep.shape[0]
 
             if total_replies >= max_replies:
-                break
                 print('The desired maximum amount of replies was reached.')
+                break
 
         if save_final:
             df_tweets_rep.to_csv(f'{filename}_replies_{self.timestamp}', index=False)
@@ -366,37 +392,37 @@ class TweetDownloader:
 
         return df_tweets_rep
 
-    def build_dataframes(self):
-        # Loop that goes through the tweet pages and stores data in DataFrames
-        print('Building DataFrame from Tweet pages...')
-
-        df_tweets = pd.DataFrame()
-        df_places = pd.DataFrame()
-        df_authors = pd.DataFrame()
-
-        for page in self.tweets:
-            df_page = pd.DataFrame(page[0]['data'])
-            df_page_authors = pd.DataFrame(page[0]['includes']['users'])
-
-            try:
-                df_page_places = pd.DataFrame(page[0]['includes']['places'])
-                df_places = pd.concat([df_places, df_page_places])
-            except KeyError:
-                print('No places on this page...')
-                print('Building DataFrame from Tweet pages...')
-            df_tweets = pd.concat([df_tweets, df_page])
-            df_authors = pd.concat([df_authors, df_page_authors])
-
-        # resets index of dataframes to avoid redundancy after concatenation
-        df_tweets.reset_index(drop=True, inplace=True)
-        df_places.reset_index(drop=True, inplace=True)
-        df_authors.reset_index(drop=True, inplace=True)
-
-        self.tweets_df = df_tweets
-        self.places_df = df_places
-        self.authors_df = df_authors
-
-        print('Done')
+#    def build_dataframes(self):
+#        # Loop that goes through the tweet pages and stores data in DataFrames
+#        print('Building DataFrame from Tweet pages...')
+#
+#        df_tweets = pd.DataFrame()
+#        df_places = pd.DataFrame()
+#        df_authors = pd.DataFrame()
+#
+#        for page in self.tweets:
+#            df_page = pd.DataFrame(page[0]['data'])
+#            df_page_authors = pd.DataFrame(page[0]['includes']['users'])
+#
+#            try:
+#                df_page_places = pd.DataFrame(page[0]['includes']['places'])
+#                df_places = pd.concat([df_places, df_page_places])
+#            except KeyError:
+#                print('No places on this page...')
+#                print('Building DataFrame from Tweet pages...')
+#            df_tweets = pd.concat([df_tweets, df_page])
+#            df_authors = pd.concat([df_authors, df_page_authors])
+#
+#        # resets index of dataframes to avoid redundancy after concatenation
+#        df_tweets.reset_index(drop=True, inplace=True)
+#        df_places.reset_index(drop=True, inplace=True)
+#        df_authors.reset_index(drop=True, inplace=True)
+#
+#        self.tweets_df = df_tweets
+#        self.places_df = df_places
+#        self.authors_df = df_authors
+#
+#        print('Done')
 
     def tweets_from_csv(self, path, sep=',', save_temp=True):
         """
@@ -415,8 +441,8 @@ class TweetDownloader:
 
         # storing parameters into variables to be passed to the query obj
         query = df_param.query('parameter=="query"')['value'].item()
-        start_time = df_param.query('parameter=="start_time"')['value'].item()
-        end_time = df_param.query('parameter=="end_time"')['value'].item()
+        start_time = validate_date(df_param.query('parameter=="start_time"')['value'].item())
+        end_time = validate_date(df_param.query('parameter=="end_time"')['value'].item())
         max_tweets_total = int(df_param.query('parameter=="max_tweets"')['value'].item())
         max_tweets_page = int(df_param.query('parameter=="max_tweets_page"')['value'].item())
         filename = df_param.query('parameter=="filename"')['value'].item()
@@ -430,6 +456,8 @@ class TweetDownloader:
 
         self.name = filename
 
+        if pd.isna(start_time): start_time = (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%dT%H:%M:%Sz")
+        if pd.isna(end_time): end_time = datetime.now().strftime("%Y-%m-%dT%H:%M:%SZ")
         if pd.isna(place): place = None
         if pd.isna(lang): lang = None
 
@@ -473,31 +501,29 @@ class TweetDownloader:
             self.wordcloud(custom_stopwords=stopwords_list, save_wordcloud=True,
                            bar_plot=plot_barplot, save_bar_plot=True)
 
-    def tweets_to_shp(self, save_path='', geo_type='centroids'):
+    def tweets_to_gdf(self, geo_type='centroids'):
         """
         Parameters
         ----------
-        path : str
-            The path to save the exported shapefile (default is current path)
         geo_type : {'centroids', 'bbox'}, optional
             The type of geometry (default is centroids)
         """
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
-        tgeo.save_tweets_shp(save_path, geo_type)
+        gdf_tweets = tgeo.get_tweets_gdf(geo_type)
+        return gdf_tweets
 
-    def places_to_shp(self, save_path='', geo_type='centroids'):
+    def places_to_gdf(self, geo_type='centroids'):
         """
         Parameters
         ----------
-        path : str
-            The path to save the exported shapefile (default is current path)
         geo_type : {'centroids', 'bbox'}, optional
             The type of geometry (default is centroids)
         """
         tgeo = TweetGeoGenerator(self)
         tgeo.create_gdf()
-        tgeo.save_places_shp(save_path, geo_type)
+        gdf_places = tgeo.get_places_gdf(geo_type)
+        return gdf_places
 
     def preview_tweet_locations(self):
         tgeo = TweetGeoGenerator(self)
@@ -566,8 +592,8 @@ class TweetDownloader:
             custom_stopwords = []
         plt.rcParams.update({'font.size': 12})
 
-        #custom_stopwords.append('http')
-        #custom_stopwords.append('https')
+        # custom_stopwords.append('http')
+        # custom_stopwords.append('https')
 
         df_tweets = self.tweets_df.copy()
 
@@ -596,7 +622,7 @@ class TweetDownloader:
             plt.savefig(os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
             print('Wordcloud saved at', os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
         else:
-            plt.show()
+            plt.show(block=False)
 
         if bar_plot:
             # creates a dataframe to handle wordcount retrieved from wordcloud
@@ -613,12 +639,16 @@ class TweetDownloader:
             plt.ylabel("word", fontsize=50)
             plt.savefig(os.path.join(self.output_folder, f'{self.name}_wordcloud.png'))
 
+            df_bar_plot = df_wordcount[:20].copy()
+
             if save_bar_plot:
-                sns.barplot(df_wordcount.freq[:20],
-                            df_wordcount.word[:20]).get_figure().savefig(os.path.join(self.output_folder,
+                sns.barplot(data=df_bar_plot,
+                            x='word',
+                            y='freq').get_figure().savefig(os.path.join(self.output_folder,
                                                                                       f'{self.name}_barplot.png'))
                 print('Barplot saved at', os.path.join(self.output_folder, f'{self.name}_barplot.png'))
             else:
-                sns.barplot(df_wordcount.freq[:20],
-                            df_wordcount.word[:20]).get_figure()
-                plt.show()
+                sns.barplot(data=df_bar_plot,
+                            x='word',
+                            y='freq').get_figure()
+                plt.show(block=False)
